@@ -16,6 +16,7 @@
  ***************************************************************************/
 
 #include "agilent_82350b.h"
+#include <linux/delay.h>
 #include <linux/ioport.h>
 #include <linux/sched.h>
 #include <linux/module.h>
@@ -33,6 +34,8 @@ int agilent_82350b_accel_attach(gpib_board_t *board, gpib_board_config_t config)
 
 void agilent_82350b_detach( gpib_board_t *board );
 
+static const char *driver_name = "agilent_82350b";
+
 int read_transfer_counter(agilent_82350b_private_t *a_priv)
 {
 	int lo, mid, value;
@@ -42,7 +45,7 @@ int read_transfer_counter(agilent_82350b_private_t *a_priv)
 	value = ~(value - 1) & 0x7fff;
 	return value;
 }
-	
+
 void set_transfer_counter(agilent_82350b_private_t *a_priv, int count)
 {
 	int complement = -count;
@@ -81,17 +84,19 @@ int agilent_82350b_go_to_standby( gpib_board_t *board )
 void agilent_82350b_request_system_control( gpib_board_t *board, int request_control )
 {
 	agilent_82350b_private_t *a_priv = board->private_data;
-	
+
 	if(request_control)
 	{
 		a_priv->card_mode_bits |= CM_SYSTEM_CONTROLLER_BIT;
-		writeb(IC_SYSTEM_CONTROLLER_BIT, a_priv->gpib_base + INTERNAL_CONFIG_REG); 
+		if(a_priv->model != MODEL_82350A)
+			writeb(IC_SYSTEM_CONTROLLER_BIT, a_priv->gpib_base + INTERNAL_CONFIG_REG);
 	}else
 	{
 		a_priv->card_mode_bits &= ~CM_SYSTEM_CONTROLLER_BIT;
-		writeb(0, a_priv->gpib_base + INTERNAL_CONFIG_REG); 
+		if(a_priv->model != MODEL_82350A)
+			writeb(0, a_priv->gpib_base + INTERNAL_CONFIG_REG);
 	}
-	writeb(a_priv->card_mode_bits, a_priv->gpib_base + CARD_MODE_REG); 
+	writeb(a_priv->card_mode_bits, a_priv->gpib_base + CARD_MODE_REG);
 	tms9914_request_system_control(board, &a_priv->tms9914_priv, request_control);
 }
 void agilent_82350b_interface_clear( gpib_board_t *board, int assert )
@@ -164,7 +169,7 @@ unsigned int agilent_82350b_t1_delay( gpib_board_t *board, unsigned int nanosec
 	agilent_82350b_private_t *a_priv = board->private_data;
 	static const int nanosec_per_clock = 30;
 	unsigned value;
-	
+
 	tms9914_t1_delay(board, &a_priv->tms9914_priv, nanosec);
 
 	value = (nanosec + nanosec_per_clock - 1) / nanosec_per_clock;
@@ -250,10 +255,102 @@ void agilent_82350b_free_private( gpib_board_t *board )
 	}
 }
 
-int agilent_82350b_generic_attach(gpib_board_t *board, int use_fifos)
+int init_82350a_hardware(gpib_board_t *board, const gpib_board_config_t *config)
+{
+	agilent_82350b_private_t *a_priv = board->private_data;
+	static const unsigned firmware_length = 5302;
+	unsigned borg_status;
+	static const unsigned timeout = 1000;
+	int i, j;
+	const char *firmware_data = config->init_data;
+	const unsigned plx_cntrl_static_bits = PLX9050_WAITO_NOT_USER0_SELECT_BIT |
+		PLX9050_USER0_OUTPUT_BIT |
+		PLX9050_LLOCK_NOT_USER1_SELECT_BIT |
+		PLX9050_USER1_OUTPUT_BIT |
+		PLX9050_USER2_OUTPUT_BIT |
+		PLX9050_USER3_OUTPUT_BIT |
+		PLX9050_PCI_READ_MODE_BIT |
+		PLX9050_PCI_WRITE_MODE_BIT |
+		PLX9050_PCI_RETRY_DELAY_BITS(64) |
+		PLX9050_DIRECT_SLAVE_LOCK_ENABLE_BIT;
+	// load borg data
+	borg_status = readb(a_priv->borg_base);
+	if((borg_status & BORG_DONE_BIT)) return 0;
+	// need to programme borg
+	if(config->init_data == NULL ||  config->init_data_length != firmware_length)
+	{
+		printk("%s: the 82350a requires a firmware after powering on.\n", driver_name);
+		return -EIO;
+	}
+	printk("%s: Loading firmware... ", driver_name);
+
+	// tickle the borg
+	writel(plx_cntrl_static_bits | PLX9050_USER3_DATA_BIT, a_priv->plx_base + PLX9050_CNTRL_REG);
+	msleep(1);
+	writel(plx_cntrl_static_bits, a_priv->plx_base + PLX9050_CNTRL_REG);
+	msleep(1);
+	writel(plx_cntrl_static_bits | PLX9050_USER3_DATA_BIT, a_priv->plx_base + PLX9050_CNTRL_REG);
+	msleep(1);
+
+	for(i = 0; i < config->init_data_length; ++i)
+	{
+		for(j = 0; j < timeout && (readb(a_priv->borg_base) & BORG_READY_BIT) == 0; ++j)
+		{
+			if(need_resched()) schedule();
+			udelay(10);
+		}
+		if(j == timeout)
+		{
+			printk("%s: timed out loading firmware.\n", driver_name);
+			return -ETIMEDOUT;
+		}
+		writeb(firmware_data[i], a_priv->gpib_base + CONFIG_DATA_REG);
+	}
+	for(j = 0; j < timeout && (readb(a_priv->borg_base) & BORG_DONE_BIT) == 0; ++j)
+	{
+		if(need_resched()) schedule();
+		udelay(10);
+	}
+	if(j == timeout)
+	{
+		printk("%s: timed out waiting for firmware load to complete.\n", driver_name);
+		return -ETIMEDOUT;
+	}
+	printk("done.\n");
+	return 0;
+}
+
+int test_sram(gpib_board_t *board)
+{
+	agilent_82350b_private_t *a_priv = board->private_data;
+	unsigned i;
+	const unsigned sram_length = pci_resource_len(a_priv->pci_device, SRAM_82350A_REGION);
+	// test SRAM
+	const unsigned byte_mask = 0xff;
+	for(i = 0; i < sram_length; ++i)
+	{
+		writeb(i & byte_mask, a_priv->sram_base + i);
+		if(need_resched()) schedule();
+	}
+	for(i = 0; i < sram_length; ++i)
+	{
+		unsigned read_value = readb(a_priv->sram_base + i);
+		if((i & byte_mask) != read_value)
+		{
+			printk("%s: SRAM test failed at %d wanted %d got %d\n",
+				driver_name, i, (i & byte_mask), read_value);
+			return -EIO;
+		}
+		if(need_resched()) schedule();
+	}
+	return 0;
+}
+
+int agilent_82350b_generic_attach(gpib_board_t *board, const gpib_board_config_t *config, int use_fifos)
 {
 	agilent_82350b_private_t *a_priv;
 	tms9914_private_t *tms_priv;
+	int retval;
 
 	board->status = 0;
 
@@ -268,54 +365,100 @@ int agilent_82350b_generic_attach(gpib_board_t *board, int use_fifos)
 	// find board
 	a_priv->pci_device = gpib_pci_get_device(board, PCI_VENDOR_ID_AGILENT,
 		PCI_DEVICE_ID_82350B, NULL);
-	if(a_priv->pci_device == NULL)
+	if(a_priv->pci_device)
 	{
-		printk("gpib: no 82350B board found\n");
-		return -ENODEV;
+		a_priv->model = MODEL_82350B;
+	}else
+	{
+		a_priv->pci_device = gpib_pci_get_subsys(board, PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9050,
+			PCI_VENDOR_ID_HP,  PCI_SUBDEVICE_ID_82350A, a_priv->pci_device);
+		if(a_priv->pci_device)
+		{
+			a_priv->model = MODEL_82350A;
+		}else
+		{
+			printk("agilent_82350b: no 82350 board found\n");
+			return -ENODEV;
+		}
 	}
-	if(pci_request_regions(a_priv->pci_device, "agilent_82350b"))
+	if(pci_enable_device(a_priv->pci_device))
+	{
+		printk("error enabling pci device\n");
 		return -EIO;
-	a_priv->gpib_base = ioremap(pci_resource_start(a_priv->pci_device, GPIB_REGION),
-		pci_resource_len(a_priv->pci_device, GPIB_REGION));
-	printk("%s: gpib base address remapped to 0x%p\n", __FUNCTION__, a_priv->gpib_base );
-	tms_priv->iobase = a_priv->gpib_base + TMS9914_BASE_REG;
-	a_priv->sram_base = ioremap(pci_resource_start(a_priv->pci_device, SRAM_REGION),
-		pci_resource_len(a_priv->pci_device, SRAM_REGION));
-	printk("%s: sram base address remapped to 0x%p\n", __FUNCTION__, a_priv->sram_base );
-	a_priv->misc_base = ioremap(pci_resource_start(a_priv->pci_device, MISC_REGION),
-		pci_resource_len(a_priv->pci_device, MISC_REGION));
-	printk("%s: misc base address remapped to 0x%p\n", __FUNCTION__, a_priv->misc_base );
+	}
+	if(pci_request_regions(a_priv->pci_device, driver_name))
+		return -EIO;
+	switch(a_priv->model)
+	{
+	case MODEL_82350A:
+		a_priv->plx_base = ioremap(pci_resource_start(a_priv->pci_device, PLX_MEM_REGION),
+			pci_resource_len(a_priv->pci_device, PLX_MEM_REGION));
+		printk("%s: plx base address remapped to 0x%p\n", __FUNCTION__, a_priv->plx_base );
+		a_priv->gpib_base = ioremap(pci_resource_start(a_priv->pci_device, GPIB_82350A_REGION),
+			pci_resource_len(a_priv->pci_device, GPIB_82350A_REGION));
+		printk("%s: gpib base address remapped to 0x%p\n", __FUNCTION__, a_priv->gpib_base );
+		tms_priv->iobase = a_priv->gpib_base + TMS9914_BASE_REG;
+		a_priv->sram_base = ioremap(pci_resource_start(a_priv->pci_device, SRAM_82350A_REGION),
+			pci_resource_len(a_priv->pci_device, SRAM_82350A_REGION));
+		printk("%s: sram base address remapped to 0x%p\n", __FUNCTION__, a_priv->sram_base );
+		a_priv->borg_base = ioremap(pci_resource_start(a_priv->pci_device, BORG_82350A_REGION),
+			pci_resource_len(a_priv->pci_device, BORG_82350A_REGION));
+		printk("%s: borg base address remapped to 0x%p\n", __FUNCTION__, a_priv->borg_base );
+		retval = init_82350a_hardware(board, config);
+		if(retval < 0) return retval;
+		break;
+	case MODEL_82350B:
+		a_priv->gpib_base = ioremap(pci_resource_start(a_priv->pci_device, GPIB_REGION),
+			pci_resource_len(a_priv->pci_device, GPIB_REGION));
+		printk("%s: gpib base address remapped to 0x%p\n", __FUNCTION__, a_priv->gpib_base );
+		tms_priv->iobase = a_priv->gpib_base + TMS9914_BASE_REG;
+		a_priv->sram_base = ioremap(pci_resource_start(a_priv->pci_device, SRAM_REGION),
+			pci_resource_len(a_priv->pci_device, SRAM_REGION));
+		printk("%s: sram base address remapped to 0x%p\n", __FUNCTION__, a_priv->sram_base );
+		a_priv->misc_base = ioremap(pci_resource_start(a_priv->pci_device, MISC_REGION),
+			pci_resource_len(a_priv->pci_device, MISC_REGION));
+		printk("%s: misc base address remapped to 0x%p\n", __FUNCTION__, a_priv->misc_base );
+		break;
+	default:
+		BUG();
+	}
 
-	if(request_irq(a_priv->pci_device->irq, agilent_82350b_interrupt, IRQF_SHARED, "agilent_82350b", board))
+	retval = test_sram(board);
+	if(retval < 0) return retval;
+
+	if(request_irq(a_priv->pci_device->irq, agilent_82350b_interrupt, IRQF_SHARED, driver_name, board))
 	{
 		printk("gpib: can't request IRQ %d\n", a_priv->pci_device->irq);
 		return -EIO;
 	}
 	a_priv->irq = a_priv->pci_device->irq;
 	printk( "agilent_82350b: IRQ %d\n", a_priv->irq );
-	if(pci_enable_device(a_priv->pci_device))
+
+	writeb(0, a_priv->gpib_base + SRAM_ACCESS_CONTROL_REG);
+	a_priv->card_mode_bits = ENABLE_PCI_IRQ_BIT;
+	writeb(a_priv->card_mode_bits, a_priv->gpib_base + CARD_MODE_REG);
+
+	if(a_priv->model == MODEL_82350A)
 	{
-		printk("error enabling pci device\n");
-		return -EIO;
+		// enable PCI interrupts for 82350a
+		writel(PLX9050_LINTR1_EN_BIT | PLX9050_LINTR2_POLARITY_BIT | PLX9050_PCI_INTR_EN_BIT,
+			a_priv->plx_base + PLX9050_INTCSR_REG);
 	}
 	
-	writeb(0, a_priv->gpib_base + SRAM_ACCESS_CONTROL_REG); 
-	a_priv->card_mode_bits = ENABLE_PCI_IRQ_BIT;
-	writeb(a_priv->card_mode_bits, a_priv->gpib_base + CARD_MODE_REG); 
 	if(use_fifos)
 	{
 		writeb(ENABLE_BUFFER_END_EVENTS_BIT | ENABLE_TERM_COUNT_EVENTS_BIT,
 			a_priv->gpib_base + EVENT_ENABLE_REG);
-		writeb(ENABLE_TERM_COUNT_INTERRUPT_BIT | ENABLE_BUFFER_END_INTERRUPT_BIT | ENABLE_TMS9914_INTERRUPTS_BIT, 
-			a_priv->gpib_base + INTERRUPT_ENABLE_REG); 
+		writeb(ENABLE_TERM_COUNT_INTERRUPT_BIT | ENABLE_BUFFER_END_INTERRUPT_BIT | ENABLE_TMS9914_INTERRUPTS_BIT,
+			a_priv->gpib_base + INTERRUPT_ENABLE_REG);
 		//write-clear event status bits
-		writeb(BUFFER_END_STATUS_BIT | TERM_COUNT_STATUS_BIT, 
+		writeb(BUFFER_END_STATUS_BIT | TERM_COUNT_STATUS_BIT,
 			a_priv->gpib_base + EVENT_STATUS_REG);
 	}else
 	{
 		writeb(0, a_priv->gpib_base + EVENT_ENABLE_REG);
-		writeb(ENABLE_TMS9914_INTERRUPTS_BIT, 
-			a_priv->gpib_base + INTERRUPT_ENABLE_REG); 
+		writeb(ENABLE_TMS9914_INTERRUPTS_BIT,
+			a_priv->gpib_base + INTERRUPT_ENABLE_REG);
 	}
 	board->t1_nano_sec = agilent_82350b_t1_delay(board, 2000);
 	tms9914_board_reset(tms_priv);
@@ -327,14 +470,14 @@ int agilent_82350b_generic_attach(gpib_board_t *board, int use_fifos)
 
 int agilent_82350b_unaccel_attach(gpib_board_t *board, gpib_board_config_t config)
 {
-	return agilent_82350b_generic_attach(board, 0);
+	return agilent_82350b_generic_attach(board, &config, 0);
 }
 
 int agilent_82350b_accel_attach(gpib_board_t *board, gpib_board_config_t config)
 {
-	return agilent_82350b_generic_attach(board, 1);
+	return agilent_82350b_generic_attach(board, &config, 1);
 }
-	
+
 void agilent_82350b_detach(gpib_board_t *board)
 {
 	agilent_82350b_private_t *a_priv = board->private_data;
@@ -342,6 +485,9 @@ void agilent_82350b_detach(gpib_board_t *board)
 
 	if(a_priv)
 	{
+		if (a_priv->plx_base) // disable interrupts
+			writel(0, a_priv->plx_base + PLX9050_INTCSR_REG);
+
 		tms_priv = &a_priv->tms9914_priv;
 		if(a_priv->irq)
 		{
@@ -350,9 +496,11 @@ void agilent_82350b_detach(gpib_board_t *board)
 		if(a_priv->gpib_base)
 		{
 			tms9914_board_reset(tms_priv);
-			iounmap((void *)a_priv->misc_base);
-			iounmap((void *)a_priv->sram_base);
-			iounmap((void *)a_priv->gpib_base);
+			if(a_priv->misc_base) iounmap((void *)a_priv->misc_base);
+			if(a_priv->borg_base) iounmap((void *)a_priv->borg_base);
+			if(a_priv->sram_base) iounmap((void *)a_priv->sram_base);
+			if(a_priv->gpib_base) iounmap((void *)a_priv->gpib_base);
+			if(a_priv->plx_base) iounmap((void *)a_priv->plx_base);
 			pci_release_regions(a_priv->pci_device);
 		}
 		if(a_priv->pci_device)
@@ -361,8 +509,9 @@ void agilent_82350b_detach(gpib_board_t *board)
 	agilent_82350b_free_private( board );
 }
 
-static const struct pci_device_id agilent_82350b_pci_table[] = 
+static const struct pci_device_id agilent_82350b_pci_table[] =
 {
+	{ PCI_VENDOR_ID_PLX, PCI_DEVICE_ID_PLX_9050, PCI_VENDOR_ID_HP,  PCI_SUBDEVICE_ID_82350A, 0, 0, 0 },
 	{ PCI_VENDOR_ID_AGILENT, PCI_DEVICE_ID_82350B, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
 	{ 0 }
 };
