@@ -86,13 +86,13 @@ static int init_gpib_file_private( gpib_file_private_t *priv )
 	}
 	init_gpib_descriptor( priv->descriptors[ 0 ] );
 	priv->descriptors[ 0 ]->is_board = 1;
-	init_MUTEX(&priv->descriptors_mutex);
+	mutex_init(&priv->descriptors_mutex);
 	return 0;
 }
 
 int ibopen(struct inode *inode, struct file *filep)
 {
-	unsigned int minor = MINOR(inode->i_rdev);
+	unsigned int minor = iminor(inode);
 	gpib_board_t *board;
 	gpib_file_private_t *priv;
 
@@ -142,7 +142,7 @@ int ibopen(struct inode *inode, struct file *filep)
 
 int ibclose(struct inode *inode, struct file *filep)
 {
-	unsigned int minor = MINOR(inode->i_rdev);
+	unsigned int minor = iminor(inode);
 	gpib_board_t *board;
 	gpib_file_private_t *priv = filep->private_data;
 
@@ -160,7 +160,7 @@ int ibclose(struct inode *inode, struct file *filep)
 	{
 		cleanup_open_devices( priv, board );
 		if( priv->holding_mutex )
-			up( &board->mutex );
+			mutex_unlock( &board->mutex );
 
 		if(priv->got_module && board->use_count)
 		{
@@ -176,11 +176,12 @@ int ibclose(struct inode *inode, struct file *filep)
 
 
 
-int ibioctl(struct inode *inode, struct file *filep, unsigned int cmd, unsigned long arg)
+long ibioctl(struct file *filep, unsigned int cmd, unsigned long arg)
 {
-	unsigned int minor = MINOR(inode->i_rdev);
+	unsigned int minor = iminor(filep->f_dentry->d_inode);
 	gpib_board_t *board;
 	gpib_file_private_t *file_priv = filep->private_data;
+	long retval = -ENOTTY;
 
 	if( minor >= GPIB_MAX_NUM_BOARDS )
 	{
@@ -189,6 +190,8 @@ int ibioctl(struct inode *inode, struct file *filep, unsigned int cmd, unsigned
 	}
 	board = &board_array[ minor ];
 
+	lock_kernel();
+
 	GPIB_DPRINTK( "pid %i, minor %i, ioctl %d, interface=%s, use=%d, onl=%d\n",
 		current->pid, minor, cmd & 0xff,
 		board->interface ? board->interface->name : "",
@@ -198,10 +201,12 @@ int ibioctl(struct inode *inode, struct file *filep, unsigned int cmd, unsigned
 	switch( cmd )
 	{
 		case CFCBOARDTYPE:
-			return board_type_ioctl(file_priv, board, arg);
+			retval = board_type_ioctl(file_priv, board, arg);
+			goto done;
 			break;
 		case IBONL:
-			return online_ioctl( board, arg );
+			retval = online_ioctl( board, arg );
+			goto done;
 			break;
 		default:
 			break;
@@ -209,14 +214,16 @@ int ibioctl(struct inode *inode, struct file *filep, unsigned int cmd, unsigned
 	if( board->interface == NULL )
 	{
 		printk("gpib: no gpib board configured on /dev/gpib%i\n", minor);
-		return -ENODEV;
+		retval = -ENODEV;
+		goto done;
 	}
 	if(file_priv->got_module == 0)
 	{
 		if(!try_module_get(board->provider_module))
 		{
 			printk("gpib: try_module_get() failed\n");
-			return -ENOSYS;
+			retval = -ENOSYS;
+			goto done;
 		}
 		file_priv->got_module = 1;
 		board->use_count++;
@@ -224,31 +231,40 @@ int ibioctl(struct inode *inode, struct file *filep, unsigned int cmd, unsigned
 	switch( cmd )
 	{
 		case CFCBASE:
-			return iobase_ioctl( board, arg );
+			retval = iobase_ioctl( board, arg );
+			goto done;
 			break;
 		case CFCIRQ:
-			return irq_ioctl( board, arg );
+			retval = irq_ioctl( board, arg );
+			goto done;
 			break;
 		case CFCDMA:
-			return dma_ioctl( board, arg );
+			retval = dma_ioctl( board, arg );
+			goto done;
 			break;
 		case IBAUTOSPOLL:
-			return autospoll_ioctl(board, arg);
+			retval = autospoll_ioctl(board, arg);
+			goto done;
 			break;
 		case IBBOARD_INFO:
-			return board_info_ioctl( board, arg );
+			retval = board_info_ioctl( board, arg );
+			goto done;
 			break;
 		case IBMUTEX:
-			return mutex_ioctl( board, file_priv, arg );
+			retval = mutex_ioctl( board, file_priv, arg );
+			goto done;
 			break;
 		case IBPAD:
-			return pad_ioctl( board, file_priv, arg );
+			retval = pad_ioctl( board, file_priv, arg );
+			goto done;
 			break;
 		case IBSAD:
-			return sad_ioctl( board, file_priv, arg );
+			retval = sad_ioctl( board, file_priv, arg );
+			goto done;
 			break;
 		case IBSELECT_PCI:
-			return select_pci_ioctl( board, arg );
+			retval = select_pci_ioctl( board, arg );
+			goto done;
 			break;
 		default:
 			break;
@@ -258,32 +274,40 @@ int ibioctl(struct inode *inode, struct file *filep, unsigned int cmd, unsigned
 	{
 		printk( "gpib: ioctl %i invalid for offline board\n",
 			cmd & 0xff );
-		return -EINVAL;
+		retval = -EINVAL;
+		goto done;
 	}
 
 	switch( cmd )
 	{
 		case IBEVENT:
-			return event_ioctl( board, arg );
+			retval = event_ioctl( board, arg );
+			goto done;
 			break;
 		case IBCLOSEDEV:
-			return close_dev_ioctl( filep, board, arg );
+			retval = close_dev_ioctl( filep, board, arg );
+			goto done;
 			break;
 		case IBOPENDEV:
-			return open_dev_ioctl( filep, board, arg );
+			retval = open_dev_ioctl( filep, board, arg );
+			goto done;
 			break;
 		case IBSPOLL_BYTES:
-			return status_bytes_ioctl( board, arg );
+			retval = status_bytes_ioctl( board, arg );
+			goto done;
 			break;
 		case IBWAIT:
-			return wait_ioctl( file_priv, board, arg );
+			retval = wait_ioctl( file_priv, board, arg );
+			goto done;
 			break;
 		case IBLINES:
-			return line_status_ioctl( board, arg );
+			retval = line_status_ioctl( board, arg );
+			goto done;
 			break;
 		case IBLOC:
 			board->interface->return_to_local( board );
-			return 0;
+			retval = 0;
+			goto done;
 			break;
 		default:
 			break;
@@ -293,65 +317,85 @@ int ibioctl(struct inode *inode, struct file *filep, unsigned int cmd, unsigned
 	{
 		printk( "gpib: need to hold board lock to perform ioctl %i\n",
 			cmd & 0xff );
-		return -EPERM;
+		retval = -EPERM;
+		goto done;
 	}
 
 	switch( cmd )
 	{
 		case IB_T1_DELAY:
-			return t1_delay_ioctl( board, arg );
+			retval = t1_delay_ioctl( board, arg );
+			goto done;
 			break;
 		case IBCAC:
-			return take_control_ioctl( board, arg );
+			retval = take_control_ioctl( board, arg );
+			goto done;
 			break;
 		case IBCMD:
-			return command_ioctl( file_priv, board, arg );
+			retval = command_ioctl( file_priv, board, arg );
+			goto done;
 			break;
 		case IBEOS:
-			return eos_ioctl( board, arg );
+			retval = eos_ioctl( board, arg );
+			goto done;
 			break;
 		case IBGTS:
-			return ibgts( board );
+			retval = ibgts( board );
+			goto done;
 			break;
 		case IBPPC:
-			return ppc_ioctl( board, arg );
+			retval = ppc_ioctl( board, arg );
+			goto done;
 			break;
 		case IBQUERY_BOARD_RSV:
-			return query_board_rsv_ioctl( board, arg );
+			retval = query_board_rsv_ioctl( board, arg );
+			goto done;
 			break;
 		case IBRD:
-			return read_ioctl( file_priv, board, arg );
+			retval = read_ioctl( file_priv, board, arg );
+			goto done;
 			break;
 		case IBRPP:
-			return parallel_poll_ioctl( board, arg );
+			retval = parallel_poll_ioctl( board, arg );
+			goto done;
 			break;
 		case IBRSC:
-			return request_system_control_ioctl( board, arg );
+			retval = request_system_control_ioctl( board, arg );
+			goto done;
 			break;
 		case IBRSP:
-			return serial_poll_ioctl( board, arg );
+			retval = serial_poll_ioctl( board, arg );
+			goto done;
 			break;
 		case IBRSV:
-			return request_service_ioctl( board, arg );
+			retval = request_service_ioctl( board, arg );
+			goto done;
 			break;
 		case IBSIC:
-			return interface_clear_ioctl( board, arg );
+			retval = interface_clear_ioctl( board, arg );
+			goto done;
 			break;
 		case IBSRE:
-			return remote_enable_ioctl( board, arg );
+			retval = remote_enable_ioctl( board, arg );
+			goto done;
 			break;
 		case IBTMO:
-			return timeout_ioctl( board, arg );
+			retval = timeout_ioctl( board, arg );
+			goto done;
 			break;
 		case IBWRT:
-			return write_ioctl( file_priv, board, arg );
+			retval = write_ioctl( file_priv, board, arg );
+			goto done;
 			break;
 		default:
-			return -ENOTTY;
+			retval = -ENOTTY;
+			goto done;
 			break;
 	}
 
-	return -ENOTTY;
+done:
+	unlock_kernel();
+	return retval;
 }
 
 static int board_type_ioctl(gpib_file_private_t *file_priv, gpib_board_t *board, unsigned long arg)
@@ -437,14 +481,16 @@ static int read_ioctl( gpib_file_private_t *file_priv, gpib_board_t *board,
 	desc = handle_to_descriptor( file_priv, read_cmd.handle );
 	if( desc == NULL ) return -EINVAL;
 
+	BUG_ON(sizeof(userbuf) > sizeof(read_cmd.buffer_ptr));
+	userbuf = (uint8_t*)(unsigned long)read_cmd.buffer_ptr;
+
 	/* Check write access to buffer */
-	if(!access_ok(VERIFY_WRITE, read_cmd.buffer, read_cmd.count))
+	if(!access_ok(VERIFY_WRITE, userbuf, read_cmd.count))
 		return -EFAULT;
 
 	desc->io_in_progress = 1;
 
 	/* Read buffer loads till we fill the user supplied buffer */
-	userbuf = read_cmd.buffer;
 	remain = read_cmd.count;
 	while(remain > 0 && end_flag == 0)
 	{
@@ -498,12 +544,13 @@ static int command_ioctl( gpib_file_private_t *file_priv,
 	desc = handle_to_descriptor( file_priv, cmd.handle );
 	if( desc == NULL ) return -EINVAL;
 
+	userbuf = (uint8_t*)(unsigned long)cmd.buffer_ptr;
+
 	/* Check read access to buffer */
-	if(!access_ok(VERIFY_READ, cmd.buffer, cmd.count))
+	if(!access_ok(VERIFY_READ, userbuf, cmd.count))
 		return -EFAULT;
 
 	/* Write buffer loads till we empty the user supplied buffer */
-	userbuf = cmd.buffer;
 	remain = cmd.count;
 
 	desc->io_in_progress = 1;
@@ -554,14 +601,15 @@ static int write_ioctl(gpib_file_private_t *file_priv, gpib_board_t *board,
 	desc = handle_to_descriptor( file_priv, write_cmd.handle );
 	if( desc == NULL ) return -EINVAL;
 
+	userbuf = (uint8_t*)(unsigned long)write_cmd.buffer_ptr;
+
 	/* Check read access to buffer */
-	if(!access_ok(VERIFY_READ, write_cmd.buffer, write_cmd.count))
+	if(!access_ok(VERIFY_READ, userbuf, write_cmd.count))
 		return -EFAULT;
 
 	desc->io_in_progress = 1;
 
 	/* Write buffer loads till we empty the user supplied buffer */
-	userbuf = write_cmd.buffer;
 	remain = write_cmd.count;
 	while(remain > 0)
 	{
@@ -735,7 +783,7 @@ static int open_dev_ioctl( struct file *filep, gpib_board_t *board, unsigned lon
 	if (retval)
 		return -EFAULT;
 
-	if(down_interruptible(&file_priv->descriptors_mutex))
+	if(mutex_lock_interruptible(&file_priv->descriptors_mutex))
 	{
 		return -ERESTARTSYS;
 	}
@@ -746,7 +794,7 @@ static int open_dev_ioctl( struct file *filep, gpib_board_t *board, unsigned lon
 	file_priv->descriptors[ i ] = kmalloc( sizeof( gpib_descriptor_t ), GFP_KERNEL );
 	if( file_priv->descriptors[ i ] == NULL )
 	{
-		up(&file_priv->descriptors_mutex);
+		mutex_unlock(&file_priv->descriptors_mutex);
 		return -ENOMEM;
 	}
 	init_gpib_descriptor( file_priv->descriptors[ i ] );
@@ -754,7 +802,7 @@ static int open_dev_ioctl( struct file *filep, gpib_board_t *board, unsigned lon
 	file_priv->descriptors[ i ]->pad = open_dev_cmd.pad;
 	file_priv->descriptors[ i ]->sad = open_dev_cmd.sad;
 	file_priv->descriptors[ i ]->is_board = open_dev_cmd.is_board;
-	up(&file_priv->descriptors_mutex);
+	mutex_unlock(&file_priv->descriptors_mutex);
 
 	retval = increment_open_device_count( &board->device_list, open_dev_cmd.pad, open_dev_cmd.sad );
 	if( retval < 0 )
@@ -862,6 +910,9 @@ static int online_ioctl( gpib_board_t *board, unsigned long arg )
 	online_ioctl_t online_cmd;
 	gpib_board_config_t config;
 	int retval;
+	void *init_data = NULL;
+
+	config.init_data = NULL;
 
 	if(!capable(CAP_SYS_ADMIN))
 		return -EPERM;
@@ -874,7 +925,9 @@ static int online_ioctl( gpib_board_t *board, unsigned long arg )
 		config.init_data = vmalloc(online_cmd.init_data_length);
 		if(config.init_data == NULL)
 			return -ENOMEM;
-		retval = copy_from_user(config.init_data, online_cmd.init_data, online_cmd.init_data_length);
+		BUG_ON(sizeof(init_data) > sizeof(online_cmd.init_data_ptr));
+		init_data = (void*)(unsigned long)(online_cmd.init_data_ptr);
+		retval = copy_from_user(config.init_data, init_data, online_cmd.init_data_length);
 		if(retval)
 		{
 			vfree(config.init_data);
@@ -890,7 +943,7 @@ static int online_ioctl( gpib_board_t *board, unsigned long arg )
 		retval = ibonline(board, config);
 	else
 		retval = iboffline(board);
-	if(online_cmd.init_data)
+	if(config.init_data)
 		vfree(config.init_data);
 	return retval;
 }
@@ -1030,7 +1083,7 @@ static int request_service_ioctl( gpib_board_t *board, unsigned long arg )
 
 static int iobase_ioctl( gpib_board_t *board, unsigned long arg )
 {
-	unsigned long base_addr;
+	uint64_t base_addr;
 	int retval;
 
 	if(!capable(CAP_SYS_ADMIN))
@@ -1040,7 +1093,8 @@ static int iobase_ioctl( gpib_board_t *board, unsigned long arg )
 	if( retval )
 		return -EFAULT;
 
-	board->ibbase = (void*)(base_addr);
+	BUG_ON(sizeof(void*) > sizeof(base_addr));
+	board->ibbase = (void*)(unsigned long)(base_addr);
 
 	return 0;
 }
@@ -1088,10 +1142,6 @@ static int autospoll_ioctl(gpib_board_t *board, unsigned long arg)
 	if(retval)
 		return -EFAULT;
 
-	if(down_interruptible(&board->autopoll_mutex))
-	{
-		return -ERESTARTSYS;
-	}
 /*FIXME: should keep track of whether autospolling is on or off
  * by descriptor.  That would also allow automatic decrement
  * of autospollers when descriptors are closed. */
@@ -1109,7 +1159,6 @@ static int autospoll_ioctl(gpib_board_t *board, unsigned long arg)
 			retval = 0;
 		}
 	}
-	up( &board->autopoll_mutex );
 	return retval;
 }
 
@@ -1124,17 +1173,12 @@ static int mutex_ioctl( gpib_board_t *board, gpib_file_private_t *file_priv,
 
 	if( lock_mutex )
 	{
-		retval = down_interruptible(&board->mutex);
+		retval = mutex_lock_interruptible(&board->mutex);
 		if(retval)
 		{
 			printk("gpib: ioctl interrupted while waiting on lock\n");
 			return -ERESTARTSYS;
 		}
-		if( atomic_read( &board->mutex.count ) > 0 )
-		{
-			printk( "gpib: bug! board->mutex.count %i after lock!\n",
-				atomic_read( &board->mutex.count ) );
-		}
 		board->locking_pid = current->pid;
 		file_priv->holding_mutex = 1;
 		GPIB_DPRINTK("locked board %d mutex\n", board->minor);
@@ -1148,12 +1192,7 @@ static int mutex_ioctl( gpib_board_t *board, gpib_file_private_t *file_priv,
 		}
 		file_priv->holding_mutex = 0;
 		board->locking_pid = 0;
-		if( atomic_read( &board->mutex.count ) > 0 )
-		{
-			printk( "gpib: bug! board->mutex.count %i before releasing lock!\n",
-				atomic_read( &board->mutex.count ) );
-		}
-		up( &board->mutex );
+		mutex_unlock( &board->mutex );
 		GPIB_DPRINTK("unlocked board %i mutex\n", board->minor);
 	}
 
