@@ -37,11 +37,12 @@ struct file_operations ib_fops =
 	write: NULL,
 	readdir: NULL,
 	poll: NULL,
-	ioctl: ibioctl,
+	unlocked_ioctl: &ibioctl,
+	compat_ioctl: &ibioctl,
 	mmap: NULL,
-	open: ibopen,
+	open: &ibopen,
 	flush: NULL,
-	release: ibclose,
+	release: &ibclose,
 	fsync: NULL,
 	fasync: NULL,
 	lock: NULL,
@@ -120,9 +121,8 @@ void init_gpib_board( gpib_board_t *board )
 	board->buffer_length = 0;
 	board->status = 0;
 	init_waitqueue_head(&board->wait);
-	init_MUTEX(&board->mutex);
+	mutex_init(&board->mutex);
 	board->locking_pid = 0;
-	init_MUTEX(&board->autopoll_mutex);
 	spin_lock_init(&board->spinlock);
 	init_timer(&board->timer);
 	board->ibbase = 0;
@@ -140,7 +140,7 @@ void init_gpib_board( gpib_board_t *board )
 	board->online = 0;
 	board->autospollers = 0;
 	board->autospoll_pid = 0;
-	init_MUTEX_LOCKED(&board->autospoll_completion);
+	init_completion(&board->autospoll_completion);
 	init_event_queue(&board->event_queue);
 	board->minor = -1;
 	init_gpib_pseudo_irq(&board->pseudo_irq);
@@ -229,7 +229,7 @@ static void __exit gpib_common_exit_module( void )
 	int i;
 	for(i = 0; i < GPIB_MAX_NUM_BOARDS; ++i)
 	{
-		class_device_destroy(gpib_class, MKDEV(IBMAJOR, i));
+		device_destroy(gpib_class, MKDEV(IBMAJOR, i));
 	}
 	class_destroy(gpib_class);
 	unregister_chrdev(IBMAJOR, "gpib");
@@ -255,6 +255,25 @@ struct pci_dev* gpib_pci_get_device( const gpib_board_t *board, unsigned int ven
 	return NULL;
 }
 
+struct pci_dev* gpib_pci_get_subsys( const gpib_board_t *board, unsigned int vendor_id,
+	unsigned int device_id, unsigned ss_vendor, unsigned ss_device,
+	struct pci_dev *from)
+{
+	struct pci_dev *pci_device = from;
+
+	while((pci_device = pci_get_subsys( vendor_id, device_id, ss_vendor, ss_device, pci_device)))
+	{
+		if(board->pci_bus >=0 && board->pci_bus != pci_device->bus->number)
+			continue;
+		if(board->pci_slot >= 0 && board->pci_slot !=
+			PCI_SLOT( pci_device->devfn))
+			continue;
+		return pci_device;
+	}
+	return NULL;
+}
+
 EXPORT_SYMBOL( gpib_register_driver );
 EXPORT_SYMBOL( gpib_unregister_driver );
 EXPORT_SYMBOL( gpib_pci_get_device );
+EXPORT_SYMBOL(gpib_pci_get_subsys);
